import javax.microedition.lcdui.*;
import javax.microedition.media.*;
import javax.microedition.media.control.*;
import javax.microedition.midlet.MIDlet;
import java.io.*;
import jp.sourceforge.qrcode.codec.exception.DecodingFailedException;
import jp.sourceforge.qrcode.codec.util.*;
import jp.sourceforge.qrcode.codec.*;

public class QRMidlet
    extends MIDlet
    implements CommandListener {
  private Display mDisplay;
  
  private Form mMainForm;
  
  private Command mExitCommand, mCameraCommand;
  private Command mBackCommand, mCaptureCommand;
  
  private TextField tf;
  
  private Player mPlayer;
  private VideoControl mVideoControl;
  
  public QRMidlet() {
    mExitCommand = new Command("Exit", Command.EXIT, 0);
    mCameraCommand = new Command("Camera", Command.SCREEN, 0);
    mBackCommand = new Command("Back", Command.BACK, 0);
    mCaptureCommand = new Command("Capture", Command.SCREEN, 0);
    tf = new TextField("Decoded string", "String will go here", 30, TextField.ANY);
    
    mMainForm = new Form("QRCode Tester");
    mMainForm.addCommand(mExitCommand);
    mMainForm.append(tf);
    String supports = System.getProperty("video.snapshot.encodings");
    supports="yes";
    if (supports != null && supports.length() > 0) {
      mMainForm.append("Ready to take pictures.");
      mMainForm.addCommand(mCameraCommand);
    }
    else
      mMainForm.append("QRCode cannot use this " +
          "device to take pictures.");
    mMainForm.setCommandListener(this);
  }
  
  public String decodeImage(Image i) {
        
        int[][] image = null;
	image = QRCodeUtility.parseImage(i);
	QRCodeDecoder decoder = new QRCodeDecoder();
	String decodedString = "";
	try {
	 decodedString = decoder.decode(image);
	} catch (DecodingFailedException e) {
		e.printStackTrace();
	}
	if (decodedString!="") decodedString = ContentConverter.convert(decodedString);
        System.out.println("Decoded string is "+decodedString);
        return decodedString;
  }
  
  public void startApp() {
    mDisplay = Display.getDisplay(this);
    
    mDisplay.setCurrent(mMainForm);
  }
    
  public void pauseApp() {}
  
  public void destroyApp(boolean unconditional) {
  }
  
  public void commandAction(Command c, Displayable s) {
    if (c.getCommandType() == Command.EXIT) {
      destroyApp(true);
      notifyDestroyed();
    }
    else if (c == mCameraCommand)
      showCamera();
    else if (c == mBackCommand)
      mDisplay.setCurrent(mMainForm);
    else if (c == mCaptureCommand) {
      capture();
    }
  }

  private void showCamera() {
    try {
      mPlayer = Manager.createPlayer("capture://video");
      mPlayer.realize();
      
      mVideoControl = (VideoControl)mPlayer.getControl("VideoControl");
      
      Canvas canvas = new CameraCanvas(this, mVideoControl);
      canvas.addCommand(mBackCommand);
      canvas.addCommand(mCaptureCommand);
      canvas.setCommandListener(this);
      mDisplay.setCurrent(canvas);
      mPlayer.start();
    }
    catch (IOException ioe) { handleException(ioe); }
    catch (MediaException me) { handleException(me); }
  }
  
  
  public String oneCapture() {
      String result = "";

        try {
            /*
                If your emulator does not support getSnapShot() method,
                for debug, comment next 4 statements and uncomment 5th one 
                to use prepared QR Code image as a substitute.
            */
            /*System.out.println("Getting capture");
            byte[] raw = mVideoControl.getSnapshot(null);
            System.out.println("Creating image");
            Image image = Image.createImage(raw, 0, raw.length);*/
            Image image = Image.createImage("/test.png");

            System.out.println("Decoding image");
            result = decodeImage(image);
            Thread.sleep(100);
        } catch (Exception e) {
            result = e.toString();
        }

      mDisplay.setCurrent(mMainForm);
      
      // Shut down the player.
      mPlayer.close();
      mPlayer = null;
      mVideoControl = null;
      return result;
  }
  
  public void capture() {
      String result = oneCapture();
      if (result=="") result="COULD NOT DECODE";
      tf.setString(result);
  }
  

   private void handleException(Exception e) {
    Alert a = new Alert("Exception", e.toString(), null, null);
    a.setTimeout(Alert.FOREVER);
    mDisplay.setCurrent(a, mMainForm);
  }


}
