package jp.sourceforge.qrcode.example;
import java.io.*;
import java.net.URL;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.imageio.*;

import jp.sourceforge.qrcode.codec.*;
import jp.sourceforge.qrcode.codec.exception.DecodingFailedException;
import jp.sourceforge.qrcode.codec.util.*;

public class QRCodeDecoderSampleGUI extends JFrame implements ActionListener {
	JMenuBar menuBar;
	JMenu fileMenu;
	JMenuItem openMenu;
	JTextField url;
	JButton button;
	JFileChooser chooser;
	BufferedImage sourceImage;
	JLabel sourceImageLabel;
	DebugCanvas canvas;
	JTextArea decodedText;

	QRCodeDecoderSampleGUI() {
		System.out.println("QRCode Decoder GUI Sample");
		setSize(400,400);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		menuBar = new JMenuBar();
		//fileMenu = new JMenu("File");
		openMenu = new JMenuItem("Open Image");
		//fileMenu.add(openMenu);
		openMenu.addActionListener(this);
		menuBar.add(openMenu);
		setJMenuBar(menuBar);
		url = new JTextField(20);
		url.setText("(Or input image url here.)");
		button = new JButton("Open from URL");
		button.addActionListener(this);
		JPanel urlPanel = new JPanel();
		urlPanel.add(url);
		urlPanel.add(button);
		button = new JButton("URL");
		getContentPane().add(urlPanel, BorderLayout.NORTH);
		chooser =  new JFileChooser("摜t@CI");
		chooser.setFileFilter(new ImageFileFilter());
		setLocation(300, 200);
		url.select(0, url.getText().length());
		show();
	}
	public void actionPerformed(ActionEvent event) {
		//System.out.println(url.getText());
		if (event.getSource().equals(openMenu)) {
			chooser.showOpenDialog(this);
			if (chooser.getSelectedFile() == null)
				return ;
			try {
				sourceImage = ImageIO.read(chooser.getSelectedFile());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		else if (event.getActionCommand().equals("Open from URL")) {
			try {
			sourceImage = ImageIO.read(new URL(url.getText()));
			} catch (Exception e) {
				e.printStackTrace();
			}
			
		}
		else
			return;
		
		if (sourceImageLabel != null)
			getContentPane().remove(sourceImageLabel);
		
		sourceImageLabel = new JLabel(new ImageIcon(sourceImage));
		getContentPane().add(sourceImageLabel, BorderLayout.WEST);
		

		QRCodeDecoder decoder = new QRCodeDecoder();
		if (canvas != null) {
			getContentPane().remove(canvas);
			canvas.setImage(null);
		}
		canvas = decoder.getCanvas();
		getContentPane().add(canvas, BorderLayout.EAST);
		int[][] image = QRCodeUtility.parseImage(sourceImage);
		String decodedString = "";
		try {
			decodedString = decoder.decode(image);
		} catch (DecodingFailedException e) {
			e.printStackTrace();
		}
		//debugImageLabel = new JLabel(new ImageIcon(decoder.getCanvas().getImage()));
		if (decodedString == null || decodedString == "") decodedString = "(decoding failed)";
		decodedString = ContentConverter.convert(decodedString);
		System.out.println("\nDecode result:");
		System.out.println(decodedString);
		if (decodedText != null)
			getContentPane().remove(decodedText);
		decodedText = new JTextArea(decodedString);
		decodedText.setLineWrap(true);
		decodedText.setRows(decodedString.length() / 20 + 1);
		if (decodedString.length() < 20)
			decodedText.setColumns(decodedString.length());
		else
			decodedText.setColumns(20);
		//decodedText.setSize(sourceImageLabel.getSize().width,100);
		getContentPane().add(decodedText, BorderLayout.SOUTH);
		pack();
	
	}
	public static void main(String[] args) {
		new QRCodeDecoderSampleGUI();
	}

}
class ImageFileFilter extends javax.swing.filechooser.FileFilter {
	String[] acceptExtendsions = {"jpg","jpeg","gif","png"};

	public String getDescription() {
		return "QR Code Image files (*.jpg,*.png,*.gif,)";
	}
	public boolean accept(File f) {
		if (f.isDirectory()) 
			return true;
		
		String extension = getExtension(f);
		for (int i = 0; i < acceptExtendsions.length; i++) {
			if (extension.equals(acceptExtendsions[i]))
				return true;
		}
		return false;
			
	}
		
  String getExtension(File f) {
    String ext = null;
    String s = f.getName();
    int i = s.lastIndexOf('.');

    if (i > 0 &&  i < s.length() - 1) {
        ext = s.substring(i+1).toLowerCase();
    }
    return ext;
  }
}
